@@ -3,6 +3,10 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
+#include <libxml/parser.h>
+#include <libxml/xpath.h>
+#include <libxml/xpathInternals.h>
+
 #include <glibtop/fsusage.h>
 #include <glibtop/mountlist.h>
 #include <glibtop/mem.h>
@@ -15,7 +19,7 @@
 #include <math.h>
 #include <errno.h>
 
-#include <pcrecpp.h>
+#include "regex.h"
 
 #include <string>
 #include <vector>
@@ -39,9 +43,9 @@ namespace {
   public:
     string hostname;
     string distro_name;
-    string distro_codename;
-    string distro_release; // numerical version
-
+    string distro_release;
+    string kernel;
+    string gnome_version;
     guint64 memory_bytes;
     guint64 free_space_bytes;
 
@@ -55,6 +59,7 @@ namespace {
       this->load_memory_info();
       this->load_disk_info();
       this->load_uname_info();
+      this->load_gnome_version();
     }
 
     virtual ~SysInfo()
@@ -125,8 +130,40 @@ namespace {
       uname(&name);
 
       this->hostname = name.nodename;
-      this->distro_name = name.sysname;
-      this->distro_release = name.release;
+      this->kernel = string(name.sysname) + ' ' + name.release;
+    }
+
+
+    void load_gnome_version()
+    {
+      xmlDocPtr document;
+      xmlXPathContextPtr context;
+      const string nodes[3] = { "string(/gnome-version/platform)",
+				"string(/gnome-version/minor)",
+				"string(/gnome-version/micro)" };
+      string values[3];
+
+      if (not (document = xmlParseFile(DATADIR "/gnome-about/gnome-version.xml")))
+	return;
+
+      if (not (context = xmlXPathNewContext(document)))
+	return;
+
+      for (size_t i = 0; i != 3; ++i)
+	{
+	  xmlXPathObjectPtr xpath;
+	  xpath = xmlXPathEvalExpression(BAD_CAST nodes[i].c_str(), context);
+	  
+	  if (xpath and xpath->type == XPATH_STRING)
+	    values[i] = reinterpret_cast<const char*>(xpath->stringval);
+
+	  xmlXPathFreeObject(xpath);
+	}
+
+      xmlXPathFreeContext(context);
+      xmlFreeDoc(document);
+
+      this->gnome_version = values[0] + '.' + values[1] + '.' + values[2];
     }
   };
 
@@ -190,12 +227,15 @@ namespace {
 				    0,
 				    &status,
 				    &error)) {
-
+	string release, codename;
 	if (!error and WIFEXITED(status) and WEXITSTATUS(status) == 0) {
 	  std::istringstream input(out);
 	  this->get_value(input, this->distro_name)
-	    and this->get_value(input, this->distro_release)
-	    and this->get_value(input, this->distro_codename);
+	    and this->get_value(input, release)
+	    and this->get_value(input, codename);
+	  this->distro_release = release;
+	  if (codename != "")
+	    this->distro_release += " (" + codename + ')';
 	}
       }
 
@@ -345,25 +385,39 @@ procman_create_sysinfo_view(void)
 			   );
   gtk_frame_set_shadow_type(GTK_FRAME(distro_frame), GTK_SHADOW_NONE);
   gtk_box_pack_start(GTK_BOX(vbox), distro_frame, FALSE, FALSE, 0);
-
   alignment = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
   gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 12, 0);
   gtk_container_add(GTK_CONTAINER(distro_frame), alignment);
 
-  if (data->distro_codename != "")
-    markup = g_strdup_printf(
-			     _("Release %s (%s)"),
-			     data->distro_release.c_str(),
-			     data->distro_codename.c_str());
-  else
-    markup = g_strdup_printf(
-			     _("Release %s"),
-			     data->distro_release.c_str());
-  distro_release_label = gtk_label_new(markup);
+  GtkWidget* distro_box = gtk_hbox_new(FALSE, 12);
+  gtk_container_add(GTK_CONTAINER(alignment), distro_box);
+
+  GtkWidget* distro_inner_box = gtk_vbox_new(FALSE, 6);
+  gtk_box_pack_start(GTK_BOX(distro_box), distro_inner_box, FALSE, FALSE, 0);
+
+  if (data->distro_release != "")
+    {
+      markup = g_strdup_printf(_("Release %s"), data->distro_release.c_str());
+      distro_release_label = gtk_label_new(markup);
+      gtk_misc_set_alignment(GTK_MISC(distro_release_label), 0.0, 0.5);
+      g_free(markup);
+      gtk_box_pack_start(GTK_BOX(distro_inner_box), distro_release_label, FALSE, FALSE, 0);
+    }
+
+  markup = g_strdup_printf(_("Kernel %s"), data->kernel.c_str());
+  GtkWidget* kernel_label = gtk_label_new(markup);
+  gtk_misc_set_alignment(GTK_MISC(kernel_label), 0.0, 0.5);
   g_free(markup);
-  gtk_misc_set_alignment(GTK_MISC(distro_release_label), 0.0, 0.5);
-  gtk_misc_set_padding(GTK_MISC(distro_release_label), 6, 6);
-  gtk_container_add(GTK_CONTAINER(alignment), distro_release_label);
+  gtk_box_pack_start(GTK_BOX(distro_inner_box), kernel_label, FALSE, FALSE, 0);
+
+  if (data->gnome_version != "")
+    {
+      markup = g_strdup_printf(_("Gnome %s"), data->gnome_version.c_str());
+      GtkWidget* gnome_label = gtk_label_new(markup);
+      gtk_misc_set_alignment(GTK_MISC(gnome_label), 0.0, 0.5);
+      g_free(markup);
+      gtk_box_pack_start(GTK_BOX(distro_inner_box), gnome_label, FALSE, FALSE, 0);
+    }
 
   /* hardware section */
 
